import pytest
from unittest import TestCase
import os
from acme_news.engines.aljazeera import AljazeeraEngine
from typing import List, Dict

class TestBrowser(TestCase):

    @pytest.mark.skipif(os.getenv('BITBUCKET_BRANCH') == 'develop' or os.getenv('BITBUCKET_BRANCH') == 'master',
                        reason='Local Test')
    def test_get_news_contents_attributes(self):
        alj = AljazeeraEngine(search_query='iraq')
        results: List[Dict] = alj.parse_alj_contents()
        return self.assertEqual(10, len(results))

    @pytest.mark.skipif(os.getenv('BITBUCKET_BRANCH') == 'develop' or os.getenv('BITBUCKET_BRANCH') == 'master',
                        reason='Local Test')
    def test_get_video_links(self):
        alj = AljazeeraEngine(search_query='iraq', video=True, quantity=100, offset=4)
        results: List[Dict] = alj.parse_video_links()
        return self.assertEqual(100, len(results))

    @pytest.mark.skipif(os.getenv('BITBUCKET_BRANCH') == 'develop' or os.getenv('BITBUCKET_BRANCH') == 'master',
                        reason='Local Test')
    def test_get_news_contents(self):
        alj = AljazeeraEngine(search_query='iraq')
        results: str = alj.parse_news_content(url="https://www.aljazeera.com/news/2019/12/4/muhasasa-the-political-system-reviled-by-iraqi-protesters")
        return self.assertEqual(7896, len(results))



import pytest
from unittest import TestCase
import os
from acme_news.engines.youtube import YoutubeEngine

class TestYoutubeEngine(TestCase):
    @pytest.mark.skipif(os.getenv('BITBUCKET_BRANCH') == 'develop' or os.getenv('BITBUCKET_BRANCH') == 'master',
                        reason='Local Test')
    def test_get_query(self):
        youtube = YoutubeEngine(search_query='iraq')
        results = youtube.load_results()
        self.assertEqual(80, len(results))



#!/usr/bin/env python
import json
from acme_news.browser.acme_browser import ACMEBrowser
from typing import List, Dict
from bs4 import BeautifulSoup
from pandas.io.json import json_normalize
import re

class YoutubeEngine(ACMEBrowser):
    def __init__(self, search_query: str):
        self.results: List = []
        self.url: str = ""
        self.__set_search_query(search_query=search_query)
        self.youtube_driver = self.browser(url=self.url)
        super(YoutubeEngine, self).__init__()

    def __set_search_query(self, search_query: str):
        if not search_query:
            raise ValueError("Please provide the right search query.")

        self.url = f'https://www.youtube.com/results?search_query={search_query}'

    def load_results(self) -> List:
        soup = BeautifulSoup(self.youtube_driver.page_source, 'html.parser')
        for tag in soup.find_all('script'):
            script = str(tag)
            if script.find("responseContext") != -1:
                self.results.append(script)

        if self.results:
            youtube_content = json.loads(re.sub("<script.*.Data =", "", self.results[0].replace("<script>", "").replace("</script>", "").rstrip(";") ) ).get('contents').get('twoColumnSearchResultsRenderer').get('primaryContents').get('sectionListRenderer').get('contents')[0].get('itemSectionRenderer').get('contents')
            df = json_normalize(youtube_content)
            results = [attr.get('thumbnailOverlayToggleButtonRenderer', {}).get('untoggledServiceEndpoint', {}).get('signalServiceEndpoint',{}) for youtube_attr in df['videoRenderer.thumbnailOverlays'].values.tolist() if isinstance(youtube_attr, List) for attr in youtube_attr]
            return results





#!/usr/bin/env python
from acme_news.browser.acme_browser import ACMEBrowser
from typing import List, Dict
import re

class CNNEngine(ACMEBrowser):
    """
    DESCRIPTION
        This class inherit the web browser functionalities from the ACMEBrowser.

    FUNCTIONS
        __init__
        __build_bulk_query__
        __build_url_query__
        __set_category__
        __set_page_size__
        __parse_cnn_api_response__
        __parse_cnn_videos_endpoints__
        get_query
        get_all_queries
        get_article_title
        get_links
        get_all_links
        get_page_content
        get_all_contents
        get_video_links
        get_all_video_links

    USAGE
        Please run the tests using pytest tests
    """
    def __init__(self, url: str = "",
                 news_article: str = "",
                 page_depth: int = 0,
                 category: str = "politics",
                 api_endpoint: bool = False,
                 video: bool = False,
                 size: int = 10):

        self.url: str = url
        self.query: str = ""
        self.use_api_endpoint: bool = api_endpoint
        self.endpoint: str = "https://search.api.cnn.io/content" if self.use_api_endpoint else "https://www.cnn.com/search"
        self.video: bool = video
        self.video_enpoint: str = "https://fave.api.cnn.io/v1/video?id"
        self.vido_required_params: str = "&customer=cnn&edition=domestic&env=prod"
        self.page_depth = page_depth
        self.news_article: str = news_article
        self.size: int = size
        self.category: str = category
        self.cnn_links: List = []
        self.batch_queries: List = []
        self.cnn_driver = None
        self.categories: List[str] = ['us', 'world', 'politics', 'business', 'opinions', 'health', 'entertainment', 'style', 'travel']

        if self.url:
            self.cnn_driver = self.browser(url=self.url)

        elif self.news_article and page_depth > 0 and not self.use_api_endpoint:
            self.batch_queries = self.__build_bulk_query__()

        elif self.news_article and not self.use_api_endpoint:
            self.cnn_driver = self.browser(url=self.__build_url_query__(size=self.size,
                                                                        category=self.category)
                                           )

        super(CNNEngine, self).__init__()

    def __build_bulk_query__(self) -> List[str]:
        if not self.news_article:
            raise ValueError(f"Please provide the right news articles")

        if self.page_depth % 10 != 0:
            raise ValueError("Page depth must be a multply of 10")

        self.batch_queries = [self.__build_url_query__(page=article_page) for article_page in range(0, self.page_depth, 10)]
        return self.batch_queries

    def __build_url_query__(self, size: int = 10, category: str = "politics", page: int = 0) -> str:

        self.__set_category__(category=category)
        self.__set_page_size__(page_size=size)

        if not self.news_article:
            raise ValueError(f"Please provide the right news articles")

        if self.category == 'all':
            self.query = f"{self.endpoint}?size={self.size}&q={self.news_article}"
            return self.query

        if self.category.lower() not in self.categories:
            raise ValueError(f"The following category {self.category} is not the right category.")

        if self.use_api_endpoint and self.video:
            self.query = f"{self.endpoint}?size={self.size}&q={self.news_article}&category={self.category}&type=video"

            if self.page_depth > 0:
                self.query += f"&from={page}&page={round(page / 10) + 1}"

            return self.query

        if self.page_depth == 0:
            self.query = f"{self.endpoint}?size={self.size}&q={self.news_article}&category={self.category}"
            return self.query

        self.query = f"{self.endpoint}?size={self.size}&q={self.news_article}&category={self.category}&from={page}&page={round(page/10) +1}"
        return self.query

    def __set_category__(self, category):
        if not category:
            raise ValueError("Please provide the right category.")

        self.category = category

    def __set_page_size__(self, page_size):
        if not page_size or page_size < 1 or not isinstance(page_size, int):
            raise ValueError("Please provide the right page size")

        self.size = page_size

    def __parse_cnn_api_response__(self, response: bytes) -> List:
        results: Dict = self.transform_requests(response=response)
        if not results:
            return []

        cnn_api_results: List  = []
        for item in results.get('result'):
            current_item: Dict = {
               'region_name' : self.news_article,
                'url': item.get('url'),
                'news_date': item.get('firstPublishDate'),
                'title': item.get('headline'),
                'category': item.get('section'),
                'content': item.get('body'),
                'image_url': item.get('thumbnail')
            }
            cnn_api_results.append(current_item)

        return cnn_api_results

    def __parse_cnn_videos_endpoints__(self, response: Dict) -> List:
        results: List = []
        for item in response.get('result'):
            video_path: str = item.get('path')
            results.append("{}={}{}".format(self.video_enpoint,
                                           video_path[video_path.find(item.get('section')):],
                                           self.vido_required_params))
        return results

    def get_query(self) -> str:
        return self.query

    def get_all_queries(self) -> List[str]:
        return self.batch_queries

    def get_article_title(self) -> str:
        return self.cnn_driver.title

    def get_links(self) -> List[str]:
        return [tag.find_element_by_tag_name('a').get_attribute('href') for tag in self.cnn_driver.find_elements_by_css_selector("h3.cnn-search__result-headline")]

    def get_all_links(self) -> List[str]:
        quries: List[str] = self.__build_bulk_query__()
        for url_query in quries:
            self.cnn_driver = self.browser(url=url_query)
            self.cnn_links.extend(self.get_links())

        return self.cnn_links

    def get_page_content(self, endpoint: str = "") -> List[Dict]:

        if endpoint:
            cnn_api_endpoint = endpoint
        else:
            cnn_api_endpoint = self.__build_url_query__()

        if not self.use_api_endpoint:
            raise NotImplementedError(f"Only available for API endpoint.")

        response: bytes = self.fetch_api_request(api_endpoint=cnn_api_endpoint)
        return self.__parse_cnn_api_response__(response=response)

    def get_all_contents(self) -> List[Dict]:
        cnn_api_endpoints: List[str] = self.__build_bulk_query__()
        results: List = []
        if not self.use_api_endpoint:
            raise NotImplementedError(f"Only available for API endpoint.")

        for query in cnn_api_endpoints:
            results.extend(self.get_page_content(endpoint=query))

        return results

    def get_video_links(self, endpoint: str = "") -> List[str]:
        if not self.endpoint and not self.video:
            raise ValueError("Please enable endpoint and the video args.")

        videos_api_endpoints: str = endpoint if endpoint else self.__build_url_query__()

        response: Dict = self.transform_requests(response=self.fetch_api_request(api_endpoint=videos_api_endpoints))
        response_video_endpoints: List[str] = self.__parse_cnn_videos_endpoints__(response=response)

        results: List = []
        for api_endpoint in response_video_endpoints:
            resp: Dict = self.transform_requests(response=self.fetch_api_request(api_endpoint=api_endpoint))
            video_link = list(filter(lambda video: re.findall(r"mp4$", video.get('fileUri')), resp.get('files')))
            if not video_link:
                video_link = "None"

            video_link = video_link[0].get('fileUri', 'None')

            video_attribute: Dict = {
                'region_name': self.news_article,
                'video_url': video_link,
                'video_category': resp.get('category'),
                'news_url': f"http:{resp.get('clickbackURL')}"
            }
            results.append(video_attribute)

        return results

    def get_all_video_links(self) -> List[str]:

        cnn_api_endpoints: List[str] = self.__build_bulk_query__()
        results: List = []
        if not self.use_api_endpoint:
            raise NotImplementedError(f"Only available for API endpoint.")

        for endpoint_url in cnn_api_endpoints:
            results.extend(self.get_video_links(endpoint=endpoint_url))

        return results





#!/usr/bin/env python
from typing import List
MIDDLE_EAST_REGIONS: List[str] = [
    'IRAQ',
    'TURKEY',
    'SYRIA',
    'YEMEN',
    'SAUDI ARABIA',
    'LEBANON',
    'PALESTINE',
    'IRAN',
    'ISRAEL',
    'UNITED ARAB EMIRATES',
    'BAHRAIN',
    'JORDAN',
    'OMAN',
    'QATAR',
    'KUWAIT'
]


#!/usr/bin/env python
import pymysql
import pandas as pd
import json

# Exceptions
from json.decoder import JSONDecodeError
from pymysql.err import ProgrammingError, MySQLError

# Alias
from typing import Dict, List
from pandas.core.frame import DataFrame

class ACMEDB(object):

    def __init__(self, credentials: str = ""):

        if not credentials:
            raise FileNotFoundError("Please provide the right credential.")

        self.credentials: Dict = self.__load_credentials__(credentials=credentials)
        self.mysql_user: str = self.credentials.get('MYSQL_USER')
        self.mysql_passwd: str = self.credentials.get('MYSQL_PASSWD')
        self.mysql_db: str = self.credentials.get("MYSQL_DB")
        self.mysql_host: str = self.credentials.get("MYSQL_SERVER")
        self.mysql_conn = pymysql.connect(host=self.mysql_host,
                                          user=self.mysql_user,
                                          passwd=self.mysql_passwd,
                                          db=self.mysql_db)
        self.mysql_cursor = self.mysql_conn.cursor()

    def __load_credentials__(self, credentials) -> Dict:

        try:
            return json.loads(open(credentials, 'r').read())

        except JSONDecodeError as e:
            raise JSONDecodeError(f"Unable to load the following credential file {self.credentials}")

    def __close__(self):
        self.mysql_conn.close()

    def __bulk_insert(self, query, items):

        if not all([query, items]):
            raise ValueError("Please provide the right query and items.")

        self.mysql_cursor.executemany(query, items)
        self.mysql_conn.commit()

    def __get_query(self, sql_query: str) -> DataFrame:
        try:
            return pd.read_sql_query(sql_query)

        except MySQLError as e:
            raise MySQLError(f"ERROR: Unable to execute the following {sql_query} SQL Statement") from e

    def insert_image_links(self, news_image_links: List[tuple]):
        if not news_image_links:
            raise ValueError("Please insert a valid image links.")

        insert_news_image_links: str = """
            INSERT INTO image_links(region_name, image_urls, image_category, news_url)
                    VALUES (%s,%s,%s,%s)
        """
        self.__bulk_insert(query=insert_news_image_links, items=news_image_links)

    def insert_news_contents(self, news_contents: List[tuple]):
        if not news_contents:
            raise ValueError("Please insert the news contents.")

        insert_news_contents: str = """
            INSERT INTO news_contents(region_name, news_urls, news_category, content)
                    VALUES (%s,%s,%s,%s)
        """
        self.__bulk_insert(query=insert_news_contents, items=news_contents)

    def insert_middle_east_regions_attributes(self, region_attributes: List[tuple]):
        if not region_attributes:
            raise ValueError("Please insert the right region attributes.")

        insert_region_attributes: str = """
            INSERT INTO middle_east_regions(region_name, url, news_date, category, title) VALUES (%s, %s, %s, %s, %s) 
            """
        self.__bulk_insert(query=insert_region_attributes, items=region_attributes)

    def insert_video_links(self, video_links: List[tuple]):
        if not video_links:
            raise ValueError("Please insert the right video links.")

        insert_video_links: str = """
            INSERT INTO video_links(region_name, video_url, video_category, news_url) 
            VALUES (%s, %s, %s, %s) 
            """
        self.__bulk_insert(query=insert_video_links, items=video_links)

    def insert_ngrams(self, ngrams_attributes: List[tuple]):
        if not ngrams_attributes:
            raise ValueError("Please insert the right ngram attributes.")

        insert_ngrams_attributes: str = """
            INSERT INTO ngrams(region_name, news_category, pickle_path) 
            VALUES (%s, %s, %s) 
            """
        self.__bulk_insert(query=insert_ngrams_attributes, items=ngrams_attributes)

    def get_news_articles(self) -> DataFrame:

        # TODO: append the video links
        news_articles_query: str = """
        SELECT 
            mer.region_name,
            mer.url,
            mer.news_date, 
            mer.category, 
            mer.title, 
            nc.content,
            il.image_urls
            
        FROM middle_east_regions mer 
        LEFT JOIN news_contents nc ON nc.news_urls = mer.url AND nc.region_name = mer.region_name 
        LEFT JOIN image_links il ON il.news_url = mer.url AND il.region_name = mer.region_name 
        """
        return self.__get_query(sql_query=news_articles_query)

    def get_video_links(self) -> DataFrame:

        video_links_query: str = """
        SELECT 
            video_id, 
            region_name,
            video_url,
            video_category,
            news_url
        FROM video_links
        """
        return self.__get_query(sql_query=video_links_query)

    def get_image_links(self) -> DataFrame:

        image_links_query: str = """
        SELECT 
            image_id, 
            region_name,
            image_urls,
            image_category,
            news_url
        FROM image_links
        """
        return self.__get_query(sql_query=image_links_query)

    def get_ngrams(self) -> DataFrame:
        search_query: str = """
        SELECT 
            id,
            region_name,
            news_category,
            pickle_path
        FROM ngrams    
        """
        return self.__get_query(sql_query=search_query)



#!/usr/bin/env python
from acme_news.browser.acme_browser import ACMEBrowser
from typing import List, Dict

class GoogleNewsEngine(ACMEBrowser):
    """
    DESCRIPTION
        This class inherit the web browser functionalities from the ACME Browser and responsible to collect news content from GOOGLE

    FUNCTIONS
    """
    def __init__(self, search_query: str = ""):
        self.google_news_url: str = ""
        self.search_query: str = search_query
        self.google_driver = None
        super(GoogleNewsEngine, self).__init__()

    def parse_google_news_links(self) -> List[Dict]:
        results: List = []

        if not self.search_query:
            raise RuntimeError("Please provide the right query.")

        self.google_news_url = f"https://news.google.com/search?q={self.search_query}&hl=en-US&gl=US&ceid=US%3Aen"
        self.google_driver = self.browser(url=self.google_news_url)
        for tag in self.google_driver.find_elements_by_tag_name("article"):
            try:
                link_attributes: Dict = {'title': tag.find_element_by_tag_name('h3').text,
                                         'link': tag.find_element_by_tag_name('a').get_attribute("href"),
                                         'date': tag.find_element_by_tag_name('time').get_attribute("datetime"),
                                         'region_name': self.search_query
                }
                results.append(link_attributes)

            except:
                pass

        return results

    def parse_news_contents(self, url: str) -> Dict:
        if not url:
            raise ValueError("Please provide the right news URLs.")

        response: bytes = self.fetch_api_request(api_endpoint=url)

        return {'url': url,
                'content': response
                }


"""Setup file for building/deploying this wheel."""
import os
import time

from setuptools import find_packages, setup

VERSION_BRANCH = ""
BITBUCKET_BRANCH: str = os.getenv("BITBUCKET_BRANCH", "")
if BITBUCKET_BRANCH and BITBUCKET_BRANCH != "master":
    VERSION_BRANCH = sum([ord(x) for x in BITBUCKET_BRANCH])

# Gather requirements from requirements.txt.  It's
# important to put all dependencies in requirements.txt rather
# then here.
#
# We're doing this because SSL v3 isn't supported by the
# setup tools dependency_links option.  Our pypi repo uses
# SSL v3 so we need to use pip (which does support SSL v3)
# to install dependencies (using requirements.txt)
install_requires = []
with open('requirements.txt') as fp:
    for line in fp:
        if not line.startswith('--') and not line.startswith('#'):
            install_requires.append(line)

# The actual call to setup tools
setup(
    name='acmenewscollectors-pkg-rioatmadja2018',
    version=f'0.1.{VERSION_BRANCH}{int(time.time())}',
    description='Python wheel to collect news articles, videos, and images.',
    long_description=open('README.md', 'r').read(),
    author='Rio Atmadja',
    author_email='rioatmadja2018@gmail.com',
    packages=find_packages(),
    install_requires=install_requires,
    setup_requires=['pytest-runner'],
    tests_require=['pytest'],
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
    ],
    zip_safe=True,
    python_requires='>=3.7'
)
